from django.db import models
from django.core.mail import get_connection, send_mail
from jsonfield import JSONField
import re

# Create your models here.

class Provider(models.Model):

    enabled = models.BooleanField(verbose_name="If this provider is enabled", default=True)
    smtp_host = models.CharField(verbose_name="Host of SMTP Server", max_length=255)
    smtp_port = models.IntegerField(verbose_name="Port of SMTP Server", default=587)
    smtp_username = models.CharField(verbose_name="User Name of SMTP Server", max_length=255)
    smtp_password = models.CharField(verbose_name="Password of SMTP Server", max_length=255)
    use_tls = models.BooleanField(verbose_name="If the server uses TLS", default=False)

    from_address = models.CharField(verbose_name="From address for this SMTP Server", max_length=255)

    blacklist = JSONField(verbose_name="Never send emails to these domains with this provider. e.g., @msn.com", blank=True)

    quota = models.IntegerField(verbose_name="Quota of the SMTP Server", default=15000)
    usage = models.IntegerField(verbose_name="Usage of the SMTP Server", default=0)
    quota_type_is_daily = models.BooleanField(verbose_name="Is the quota of the server daily?", default=False)

    preference = models.IntegerField(verbose_name="Preference of the Server, Higher the Better", default=0)

    def reset_usage(self):
        self.usage = 0
        self.save()

    def add_usage(self):
        self.usage += 1
        self.save()

    def within_quota(self):
        if not self.quota:
            return True
        return self.usage < self.quota

    def __str__(self):
        return self.smtp_host

    def __unicode__(self):
        return self.__str__()

    def get_conection(self):
        return get_connection(host=self.smtp_host,
                              port=self.smtp_port,
                              username=self.smtp_username,
                              password=self.smtp_password,
                              use_tls=self.use_tls)

    def send(self, address, title, content, html_message=None):
        send_mail(title, content, self.from_address, [address], html_message=html_message, connection=self.get_conection(), fail_silently=False)
        self.add_usage()

    def can_send(self, address):
        if not self.enabled:
            return False
        if not self.within_quota():
            return False
        suffix = re.search("@[\w.]+", address).group()
        if suffix in self.blacklist:
            return False
        return True







from django.test import TestCase
from .models import Provider


# Create your tests here.
class ProviderTest(TestCase):
    """SMTP Provider Test"""

    def setUp(self):
        self.test_provider = Provider(
            enabled=True,
            smtp_host="localhost",
            smtp_port=587,
            smtp_username="",
            smtp_password="",
            use_tls=False,
            from_address="test@example.com",
            blacklist=[]
        )

    def test_can_send_normal(self):
        """
        Test if the provider can send a normal mail
        """
        self.assertEqual(self.test_provider.can_send("test@example.com"), True)

    def test_disabled(self):
        """
        Test if the provider works when disabled
        """
        self.test_provider.enabled = False
        self.assertEqual(self.test_provider.can_send("test@example.com"), False)
        self.test_provider.enabled = True

    def test_blacklist(self):
        """
        Test if the provider can send a mail to an address in the blacklist.
        """
        self.test_provider.blacklist = ["@qq.com"]
        self.assertEqual(self.test_provider.can_send("test@qq.com"), False)

    def test_over_quota(self):
        """
        Test if the provider works when out of quota
        """
        pre_usage = self.test_provider.usage
        self.test_provider.usage = self.test_provider.quota
        self.assertEqual(self.test_provider.can_send("test@example.com"), False)
        self.test_provider.usage = pre_usage

    def test_reset_usage(self):
        """
        Test if the provider resets usage
        """
        self.test_provider.usage = 10
        self.test_provider.reset_usage()
        self.assertEqual(self.test_provider.usage, 0)

    def test_usage_count(self):
        """
        Test if the provider works with usage counting
        """
        pre_usage = self.test_provider.usage
        self.test_provider.send("test@example.com", "Test", "Test")
        post_usage = self.test_provider.usage
        self.assertEqual(post_usage - pre_usage, 1)




from celery import shared_task
from django.template.loader import render_to_string
from django.core.mail import send_mail
from django.conf import settings
from celery.schedules import crontab
from celery.task import periodic_task
from asyncmailer.models import Provider
import random


# do not use this function in the wild!
def async_render_and_send(email, title, context_dict=None, template='asyncmailer/email.html', template_plaintext='asyncmailer/email_pt.html'):
    if context_dict:
        rich_text = render_to_string(template, context_dict)
        plain_text = render_to_string(template_plaintext, context_dict)
    else:
        rich_text = render_to_string(template)
        plain_text = render_to_string(template_plaintext)

    providers = Provider.objects.all()
    good_providers = sorted([x for x in providers if x.can_send(email)], key=lambda x: x.can_send, reverse=True)
    top_providers = filter(lambda x: x.preference == good_providers[0].preference, good_providers)

    random.choice(top_providers).send(email, title, plain_text, html_message=rich_text)


@shared_task
def async_mail(email, title, context_dict=None, template='asyncmailer/email.html', template_plaintext='asyncmailer/email_pt.html'):
    if len(email) == 1:
        async_render_and_send(email[0], title, context_dict=context_dict, template=template, template_plaintext=template_plaintext)
    else:
        for address in email:
            async_render_and_send(address, title, context_dict=context_dict[address], template=template, template_plaintext=template_plaintext)


@periodic_task(run_every=crontab(hour=0))
def clear_daily_usages():
    providers = Provider.objects.fileter(quota_type_is_daily=True)
    for p in providers:
        p.reset_usage()


@periodic_task(run_every=crontab(day_of_month=1))
def clear_monthly_usages():
    providers = Provider.objects.fileter(quota_type_is_daily=False)
    for p in providers:
        p.reset_usage()



import os
from setuptools import setup
from distutils.core import Command

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from django.conf import settings
        settings.configure(DATABASES={'default': {'NAME': ':memory:',
            'ENGINE': 'django.db.backends.sqlite3'}},
            INSTALLED_APPS=('asyncmailer',))
        from django.core.management import call_command
        import django

        if django.VERSION[:2] >= (1, 7):
            django.setup()
        call_command('test', 'asyncmailer')

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-asyncmailer',
    version='0.3',
    packages=['asyncmailer'],
    include_package_data=True,
    license='MIT License',  # example license
    description='Async email solution w/ multiple SMTP credentials, load-balancing and blacklist.',
    long_description=README,
    url='https://github.com/yzmtf2008/django-asyncmailer',
    author='Andy Fang',
    author_email='andyfang.dz@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'django-jsonfield',
        'django',
        'celery'
    ],
    cmdclass={'test': TestCommand},
)
